@@ -305,6 +305,7 @@ static inline Int AVLIndex( Obj t, Int i )
 
 static Obj AVLIndex_C( Obj self, Obj t, Obj i )
 {
+    Int p;
     if (!IS_INTOBJ(i) || 
         TNUM_OBJ(t) != T_POSOBJ ||
         (TYPE_POSOBJ(t) != AVLTreeType &&
@@ -312,7 +313,7 @@ static Obj AVLIndex_C( Obj self, Obj t, Obj i )
         ErrorQuit( "Usage: AVLIndex(avltree, integer)", 0L, 0L );
         return 0L;
     }
-    Int p = AVLIndex( t, INT_INTOBJ(i) );
+    p = AVLIndex( t, INT_INTOBJ(i) );
     if (p == 0) 
         return Fail;
     else
@@ -321,6 +322,7 @@ static Obj AVLIndex_C( Obj self, Obj t, Obj i )
 
 static Obj AVLIndexFind_C( Obj self, Obj t, Obj i )
 {
+    Int p;
     if (!IS_INTOBJ(i) || 
         TNUM_OBJ(t) != T_POSOBJ ||
         (TYPE_POSOBJ(t) != AVLTreeType &&
@@ -328,7 +330,7 @@ static Obj AVLIndexFind_C( Obj self, Obj t, Obj i )
         ErrorQuit( "Usage: AVLIndexFind(avltree, integer)", 0L, 0L );
         return 0L;
     }
-    Int p = AVLIndex( t, INT_INTOBJ(i) );
+    p = AVLIndex( t, INT_INTOBJ(i) );
     if (p == 0) 
         return Fail;
     else
@@ -337,6 +339,8 @@ static Obj AVLIndexFind_C( Obj self, Obj t, Obj i )
 
 static Obj AVLIndexLookup_C( Obj self, Obj t, Obj i )
 {
+    Int p;
+    Obj vals;
     if (!IS_INTOBJ(i) || 
         TNUM_OBJ(t) != T_POSOBJ ||
         (TYPE_POSOBJ(t) != AVLTreeType &&
@@ -344,9 +348,9 @@ static Obj AVLIndexLookup_C( Obj self, Obj t, Obj i )
         ErrorQuit( "Usage: AVLIndexLookup(avltree, integer)", 0L, 0L );
         return 0L;
     }
-    Int p = AVLIndex(t,INT_INTOBJ(i));
+    p = AVLIndex(t,INT_INTOBJ(i));
     if (p == 0) return Fail;
-    Obj vals = AVLValues(t);
+    vals = AVLValues(t);
     p /= 4;
     if (vals == Fail || !ISB_LIST(vals,p))
         return True;
@@ -467,7 +471,7 @@ static inline void AVLRebalance( Obj tree, Int q, Int *newroot, int *shrink )
   *newroot = p;
 }
 
-Obj static AVLRebalance_C( Obj self, Obj tree, Obj q )
+static Obj AVLRebalance_C( Obj self, Obj tree, Obj q )
 {
     Int newroot = 0;
     int shrink;
@@ -479,7 +483,7 @@ Obj static AVLRebalance_C( Obj self, Obj tree, Obj q )
     return tmp;
 }
 
-Obj static AVLAdd_C( Obj self, Obj tree, Obj data, Obj value )
+static Obj AVLAdd_C( Obj self, Obj tree, Obj data, Obj value )
 {
 /* Parameters: tree, data, value
     tree is an AVL tree
@@ -502,6 +506,7 @@ Obj static AVLAdd_C( Obj self, Obj tree, Obj data, Obj value )
   Int c;
   Int l;
   Int i;
+  int shrink;
 
   if (TNUM_OBJ(tree) != T_POSOBJ || TYPE_POSOBJ(tree) != AVLTreeTypeMutable) {
       ErrorQuit( "Usage: AVLAdd(avltree, object, object)", 0L, 0L );
@@ -599,7 +604,7 @@ Obj static AVLAdd_C( Obj self, Obj tree, Obj data, Obj value )
   
   /* now at last we do have to rebalance at nodes[q] because the tree has
      gotten out of balance: */
-  int shrink;   /* not used */
+  shrink;   /* not used */
   AVLRebalance(tree,nodes[q],&p,&shrink);
   
   /* finishing touch: link new root of subtree (p) to t: */
@@ -614,7 +619,7 @@ Obj static AVLAdd_C( Obj self, Obj tree, Obj data, Obj value )
   return True;
 }
 
-Obj static AVLIndexAdd_C( Obj self, Obj tree, Obj data, Obj value, Obj ind )
+static Obj AVLIndexAdd_C( Obj self, Obj tree, Obj data, Obj value, Obj ind )
 {
 /* Parameters: tree, data, value
     tree is an AVL tree
@@ -636,6 +641,7 @@ Obj static AVLIndexAdd_C( Obj self, Obj tree, Obj data, Obj value, Obj ind )
   Int index;
   Int i;
   Int offset;
+  int shrink;
 
   if (TNUM_OBJ(tree) != T_POSOBJ || TYPE_POSOBJ(tree) != AVLTreeTypeMutable) {
       ErrorQuit( "Usage: AVLAdd(avltree, object, object)", 0L, 0L );
@@ -733,7 +739,7 @@ Obj static AVLIndexAdd_C( Obj self, Obj tree, Obj data, Obj value, Obj ind )
   
   /* now at last we do have to rebalance at nodes[q] because the tree has
      gotten out of balance: */
-  int shrink;   /* not used */
+  shrink;   /* not used */
   AVLRebalance(tree,nodes[q],&p,&shrink);
   
   /* finishing touch: link new root of subtree (p) to t: */
@@ -922,7 +928,7 @@ static Obj AVLDelete_C( Obj self, Obj tree, Obj data)
   return old;
 }
  
-Obj static AVLIndexDelete_C( Obj self, Obj tree, Obj index)
+static Obj AVLIndexDelete_C( Obj self, Obj tree, Obj index)
   /* Parameters: tree, index
       tree is an AVL tree
       index is the index of the element to be deleted, must be between 1 and
@@ -1348,544 +1354,6 @@ static Obj HTUpdate_TreeHash_C(Obj self, Obj ht, Obj x, Obj v)
     return old;
 }
 
-static Obj GenericHashFunc_C(Obj self, Obj x, Obj data)
-{
-     Int mult = INT_INTOBJ(ELM_PLIST(data,1));
-     UChar *p = (UChar *) ADDR_OBJ(x) + INT_INTOBJ(ELM_PLIST(data,2));
-     Int len = INT_INTOBJ(ELM_PLIST(data,3));
-     UInt mod = INT_INTOBJ(ELM_PLIST(data,4));
-     UInt n = 0;
-     Int i;
-     for (i = 0;i < len;i++) n = ((n*mult)+(UInt)(*p++));
-     return INTOBJ_INT((n % mod) + 1);
-}
-
-/* For the time being we copy the Jenkins Hash stuff here: */
-
-/******************************************************************************
- *
- *    jhash.h     The Bob Jenkins Hash Function
- *
- *    File      : $RCSfile: jhash.h,v $
- *    Author    : Henrik Bäärnhielm 
- *    Dev start : 2006-07-16 
- *
- *    Version   : $Revision: 1.7 $
- *    Date      : $Date: 2010/02/23 15:12:39 $
- *    Last edit : $Author: gap $
- *
- *    @(#)$Id: jhash.h,v 1.7 2010/02/23 15:12:39 gap Exp $
- *
- *    Definitions for Jenkins Hash
- *
- *****************************************************************************/
-
-#include <sys/types.h>
-
-#ifdef linux
-# include <endian.h>
-#endif
-
-#if !defined(__CYGWIN__)
-typedef u_int32_t uint32_t;
-typedef u_int16_t uint16_t;
-typedef u_int8_t uint8_t;
-#endif
-
-
-/******************************************************************************
- *
- *    jhash.c     The Bob Jenkins Hash Function
- *
- *    File      : $RCSfile: jhash.c,v $
- *    Author    : Bob Jenkins (cosmetic changes by Henrik Bäärnhielm) 
- *    Dev start : 2006-07-16 
- *
- *    Version   : $Revision: 1.2 $
- *    Date      : $Date: 2006/09/01 16:46:00 $
- *    Last edit : $Author: sal $
- *
- *    @(#)$Id: jhash.c,v 1.2 2006/09/01 16:46:00 sal Exp $
- *
- *    According to Prof. Viggo Kann at the CS department, Royal Institute of 
- *    Technology, Stockholm, Sweden, this is the best hash function known to
- *    humanity. It is fast and results in extremely few collisions. 
- *    Bob Jenkins original documentation is left in the code. 
- *    The file was fetched from http://burtleburtle.net/bob/c/lookup3.c
- *
- *****************************************************************************/
-
-/*
--------------------------------------------------------------------------------
-lookup3.c, by Bob Jenkins, May 2006, Public Domain.
-
-These are functions for producing 32-bit hashes for hash table lookup.
-hashword(), hashlittle(), hashbig(), mix(), and final() are externally 
-useful functions.  Routines to test the hash are included if SELF_TEST 
-is defined.  You can use this free for any purpose.  It has no warranty.
-
-You probably want to use hashlittle().  hashlittle() and hashbig()
-hash byte arrays.  hashlittle() is is faster than hashbig() on
-little-endian machines.  Intel and AMD are little-endian machines.
-
-If you want to find a hash of, say, exactly 7 integers, do
-  a = i1;  b = i2;  c = i3;
-  mix(a,b,c);
-  a += i4; b += i5; c += i6;
-  mix(a,b,c);
-  a += i7;
-  final(a,b,c);
-then use c as the hash value.  If you have a variable length array of
-4-byte integers to hash, use hashword().  If you have a byte array (like
-a character string), use hashlittle().  If you have several byte arrays, or
-a mix of things, see the comments above hashlittle().
--------------------------------------------------------------------------------
-*/
-//#define SELF_TEST 1
-
-/*
- * My best guess at if you are big-endian or little-endian.  This may
- * need adjustment.
- */
-#if (defined(__BYTE_ORDER) && defined(__LITTLE_ENDIAN) && \
-     __BYTE_ORDER == __LITTLE_ENDIAN) || \
-    (defined(i386) || defined(__i386__) || defined(__i486__) || \
-     defined(__i586__) || defined(__i686__) || defined(vax) || defined(MIPSEL))
-# define HASH_LITTLE_ENDIAN 1
-# define HASH_BIG_ENDIAN 0
-#elif (defined(__BYTE_ORDER) && defined(__BIG_ENDIAN) && \
-       __BYTE_ORDER == __BIG_ENDIAN) || \
-      (defined(sparc) || defined(POWERPC) || defined(mc68000) || defined(sel))
-# define HASH_LITTLE_ENDIAN 0
-# define HASH_BIG_ENDIAN 1
-#else
-# define HASH_LITTLE_ENDIAN 0
-# define HASH_BIG_ENDIAN 0
-#endif
-
-#define hashsize(n) ((uint32_t)1<<(n))
-#define hashmask(n) (hashsize(n)-1)
-#define rot(x,k) (((x)<<(k)) ^ ((x)>>(32-(k))))
-
-/*
--------------------------------------------------------------------------------
-mix -- mix 3 32-bit values reversibly.
-
-This is reversible, so any information in (a,b,c) before mix() is
-still in (a,b,c) after mix().
-
-If four pairs of (a,b,c) inputs are run through mix(), or through
-mix() in reverse, there are at least 32 bits of the output that
-are sometimes the same for one pair and different for another pair.
-This was tested for:
-* pairs that differed by one bit, by two bits, in any combination
-  of top bits of (a,b,c), or in any combination of bottom bits of
-  (a,b,c).
-* "differ" is defined as +, -, ^, or ~^.  For + and -, I transformed
-  the output delta to a Gray code (a^(a>>1)) so a string of 1's (as
-  is commonly produced by subtraction) look like a single 1-bit
-  difference.
-* the base values were pseudorandom, all zero but one bit set, or 
-  all zero plus a counter that starts at zero.
-
-Some k values for my "a-=c; a^=rot(c,k); c+=b;" arrangement that
-satisfy this are
-    4  6  8 16 19  4
-    9 15  3 18 27 15
-   14  9  3  7 17  3
-Well, "9 15 3 18 27 15" didn't quite get 32 bits diffing
-for "differ" defined as + with a one-bit base and a two-bit delta.  I
-used http://burtleburtle.net/bob/hash/avalanche.html to choose 
-the operations, constants, and arrangements of the variables.
-
-This does not achieve avalanche.  There are input bits of (a,b,c)
-that fail to affect some output bits of (a,b,c), especially of a.  The
-most thoroughly mixed value is c, but it doesn't really even achieve
-avalanche in c.
-
-This allows some parallelism.  Read-after-writes are good at doubling
-the number of bits affected, so the goal of mixing pulls in the opposite
-direction as the goal of parallelism.  I did what I could.  Rotates
-seem to cost as much as shifts on every machine I could lay my hands
-on, and rotates are much kinder to the top and bottom bits, so I used
-rotates.
--------------------------------------------------------------------------------
-*/
-#define mix(a,b,c) \
-{ \
-  a -= c;  a ^= rot(c, 4);  c += b; \
-  b -= a;  b ^= rot(a, 6);  a += c; \
-  c -= b;  c ^= rot(b, 8);  b += a; \
-  a -= c;  a ^= rot(c,16);  c += b; \
-  b -= a;  b ^= rot(a,19);  a += c; \
-  c -= b;  c ^= rot(b, 4);  b += a; \
-}
-
-/*
--------------------------------------------------------------------------------
-final -- final mixing of 3 32-bit values (a,b,c) into c
-
-Pairs of (a,b,c) values differing in only a few bits will usually
-produce values of c that look totally different.  This was tested for
-* pairs that differed by one bit, by two bits, in any combination
-  of top bits of (a,b,c), or in any combination of bottom bits of
-  (a,b,c).
-* "differ" is defined as +, -, ^, or ~^.  For + and -, I transformed
-  the output delta to a Gray code (a^(a>>1)) so a string of 1's (as
-  is commonly produced by subtraction) look like a single 1-bit
-  difference.
-* the base values were pseudorandom, all zero but one bit set, or 
-  all zero plus a counter that starts at zero.
-
-These constants passed:
- 14 11 25 16 4 14 24
- 12 14 25 16 4 14 24
-and these came close:
-  4  8 15 26 3 22 24
- 10  8 15 26 3 22 24
- 11  8 15 26 3 22 24
--------------------------------------------------------------------------------
-*/
-#define final(a,b,c) \
-{ \
-  c ^= b; c -= rot(b,14); \
-  a ^= c; a -= rot(c,11); \
-  b ^= a; b -= rot(a,25); \
-  c ^= b; c -= rot(b,16); \
-  a ^= c; a -= rot(c,4);  \
-  b ^= a; b -= rot(a,14); \
-  c ^= b; c -= rot(b,24); \
-}
-
-/*
---------------------------------------------------------------------
- This works on all machines.  To be useful, it requires
- -- that the key be an array of uint32_t's, and
- -- that all your machines have the same endianness, and
- -- that the length be the number of uint32_t's in the key
-
- The function hashword() is identical to hashlittle() on little-endian
- machines, and identical to hashbig() on big-endian machines,
- except that the length has to be measured in uint32_ts rather than in
- bytes.  hashlittle() is more complicated than hashword() only because
- hashlittle() has to dance around fitting the key bytes into registers.
---------------------------------------------------------------------
-*/
-uint32_t hashword(register uint32_t *k, 
-						register size_t length, 
-						register uint32_t initval)
-	/* the key, an array of uint32_t values */
-	/* the length of the key, in uint32_ts */
-	/* the previous hash, or an arbitrary value */
-{
-  register uint32_t a,b,c;
-
-  /* Set up the internal state */
-  a = b = c = 0xdeadbeef + (((uint32_t)length)<<2) + initval;
-
-  /*------------------------------------------------- handle most of the key */
-  while (length > 3)
-  {
-    a += k[0];
-    b += k[1];
-    c += k[2];
-    mix(a,b,c);
-    length -= 3;
-    k += 3;
-  }
-
-  /*------------------------------------------- handle the last 3 uint32_t's */
-  switch(length)                     /* all the case statements fall through */
-  { 
-  case 3 : c+=k[2];
-  case 2 : b+=k[1];
-  case 1 : a+=k[0];
-    final(a,b,c);
-  case 0:     /* case 0: nothing left to add */
-    break;
-  }
-  /*------------------------------------------------------ report the result */
-  return c;
-}
-
-
-/*
--------------------------------------------------------------------------------
-hashlittle() -- hash a variable-length key into a 32-bit value
-  k       : the key (the unaligned variable-length array of bytes)
-  length  : the length of the key, counting by bytes
-  initval : can be any 4-byte value
-Returns a 32-bit value.  Every bit of the key affects every bit of
-the return value.  Two keys differing by one or two bits will have
-totally different hash values.
-
-The best hash table sizes are powers of 2.  There is no need to do
-mod a prime (mod is sooo slow!).  If you need less than 32 bits,
-use a bitmask.  For example, if you need only 10 bits, do
-  h = (h & hashmask(10));
-In which case, the hash table should have hashsize(10) elements.
-
-If you are hashing n strings (uint8_t **)k, do it like this:
-  for (i=0, h=0; i<n; ++i) h = hashlittle( k[i], len[i], h);
-
-By Bob Jenkins, 2006.  bob_jenkins@burtleburtle.net.  You may use this
-code any way you wish, private, educational, or commercial.  It's free.
-
-Use for hash table lookup, or anything where one collision in 2^^32 is
-acceptable.  Do NOT use for cryptographic purposes.
--------------------------------------------------------------------------------
-*/
-
-uint32_t hashlittle( register void *key, register size_t length, 
-							register uint32_t initval)
-{
-  register uint32_t a,b,c;
-
-  /* Set up the internal state */
-  a = b = c = 0xdeadbeef + ((uint32_t)length) + initval;
-
-  if (HASH_LITTLE_ENDIAN && !((((uint8_t *)key)-(uint8_t *)0) & 0x3)) {
-    register uint32_t *k = key;                /* read 32-bit chunks */
-
-    /*------ all but last block: aligned reads and affect 32 bits of (a,b,c) */
-    while (length > 12)
-    {
-      a += k[0];
-      b += k[1];
-      c += k[2];
-      mix(a,b,c);
-      length -= 12;
-      k += 3;
-    }
-
-    /*----------------------------- handle the last (probably partial) block */
-    switch(length)
-    {
-    case 12: c+=k[2]; b+=k[1]; a+=k[0]; break;
-    case 11: c+=k[2]&0xffffff; b+=k[1]; a+=k[0]; break;
-    case 10: c+=k[2]&0xffff; b+=k[1]; a+=k[0]; break;
-    case 9 : c+=k[2]&0xff; b+=k[1]; a+=k[0]; break;
-    case 8 : b+=k[1]; a+=k[0]; break;
-    case 7 : b+=k[1]&0xffffff; a+=k[0]; break;
-    case 6 : b+=k[1]&0xffff; a+=k[0]; break;
-    case 5 : b+=k[1]&0xff; a+=k[0]; break;
-    case 4 : a+=k[0]; break;
-    case 3 : a+=k[0]&0xffffff; break;
-    case 2 : a+=k[0]&0xffff; break;
-    case 1 : a+=k[0]&0xff; break;
-    case 0 : return c;              /* zero length strings require no mixing */
-    }
-
-  } else if (HASH_LITTLE_ENDIAN && !((((uint8_t *)key)-(uint8_t *)0) & 0x1)) {
-    register uint16_t *k = key;                      /* read 16-bit chunks */
-
-    /*--------------- all but last block: aligned reads and different mixing */
-    while (length > 12)
-    {
-      a += k[0] + (((uint32_t)k[1])<<16);
-      b += k[2] + (((uint32_t)k[3])<<16);
-      c += k[4] + (((uint32_t)k[5])<<16);
-      mix(a,b,c);
-      length -= 12;
-      k += 6;
-    }
-
-    /*----------------------------- handle the last (probably partial) block */
-    switch(length)
-    {
-    case 12: c+=k[4]+(((uint32_t)k[5])<<16);
-             b+=k[2]+(((uint32_t)k[3])<<16);
-             a+=k[0]+(((uint32_t)k[1])<<16);
-             break;
-    case 11: c+=((uint32_t)(k[5]&0xff))<<16;/* fall through */
-    case 10: c+=k[4];
-             b+=k[2]+(((uint32_t)k[3])<<16);
-             a+=k[0]+(((uint32_t)k[1])<<16);
-             break;
-    case 9 : c+=k[4]&0xff;                /* fall through */
-    case 8 : b+=k[2]+(((uint32_t)k[3])<<16);
-             a+=k[0]+(((uint32_t)k[1])<<16);
-             break;
-    case 7 : b+=((uint32_t)(k[3]&0xff))<<16;/* fall through */
-    case 6 : b+=k[2];
-             a+=k[0]+(((uint32_t)k[1])<<16);
-             break;
-    case 5 : b+=k[2]&0xff;                /* fall through */
-    case 4 : a+=k[0]+(((uint32_t)k[1])<<16);
-             break;
-    case 3 : a+=((uint32_t)(k[1]&0xff))<<16;/* fall through */
-    case 2 : a+=k[0];
-             break;
-    case 1 : a+=k[0]&0xff;
-             break;
-    case 0 : return c;                     /* zero length requires no mixing */
-    }
-
-  } else {                        /* need to read the key one byte at a time */
-    register uint8_t *k = key;
-
-    /*--------------- all but the last block: affect some 32 bits of (a,b,c) */
-    while (length > 12)
-    {
-      a += k[0];
-      a += ((uint32_t)k[1])<<8;
-      a += ((uint32_t)k[2])<<16;
-      a += ((uint32_t)k[3])<<24;
-      b += k[4];
-      b += ((uint32_t)k[5])<<8;
-      b += ((uint32_t)k[6])<<16;
-      b += ((uint32_t)k[7])<<24;
-      c += k[8];
-      c += ((uint32_t)k[9])<<8;
-      c += ((uint32_t)k[10])<<16;
-      c += ((uint32_t)k[11])<<24;
-      mix(a,b,c);
-      length -= 12;
-      k += 12;
-    }
-
-    /*-------------------------------- last block: affect all 32 bits of (c) */
-    switch(length)                   /* all the case statements fall through */
-    {
-    case 12: c+=((uint32_t)k[11])<<24;
-    case 11: c+=((uint32_t)k[10])<<16;
-    case 10: c+=((uint32_t)k[9])<<8;
-    case 9 : c+=k[8];
-    case 8 : b+=((uint32_t)k[7])<<24;
-    case 7 : b+=((uint32_t)k[6])<<16;
-    case 6 : b+=((uint32_t)k[5])<<8;
-    case 5 : b+=k[4];
-    case 4 : a+=((uint32_t)k[3])<<24;
-    case 3 : a+=((uint32_t)k[2])<<16;
-    case 2 : a+=((uint32_t)k[1])<<8;
-    case 1 : a+=k[0];
-             break;
-    case 0 : return c;
-    }
-  }
-
-  final(a,b,c);
-  return c;
-}
-
-
-
-/*
- * hashbig():
- * This is the same as hashword() on big-endian machines.  It is different
- * from hashlittle() on all machines.  hashbig() takes advantage of
- * big-endian byte ordering. 
- */
-uint32_t hashbig(register void *key, 
-					  register size_t length, 
-					  register uint32_t initval)
-{
-	register uint32_t a,b,c;
-
-  /* Set up the internal state */
-  a = b = c = 0xdeadbeef + ((uint32_t)length) + initval;
-
-  if (HASH_BIG_ENDIAN && !((((uint8_t *)key)-(uint8_t *)0) & 0x3)) {
-    register uint32_t *k = key;                 /* read 32-bit chunks */
-
-    /*------ all but last block: aligned reads and affect 32 bits of (a,b,c) */
-    while (length > 12)
-    {
-      a += k[0];
-      b += k[1];
-      c += k[2];
-      mix(a,b,c);
-      length -= 12;
-      k += 3;
-    }
-
-    /*----------------------------- handle the last (probably partial) block */
-    switch(length)
-    {
-    case 12: c+=k[2]; b+=k[1]; a+=k[0]; break;
-    case 11: c+=k[2]<<8; b+=k[1]; a+=k[0]; break;
-    case 10: c+=k[2]<<16; b+=k[1]; a+=k[0]; break;
-    case 9 : c+=k[2]<<24; b+=k[1]; a+=k[0]; break;
-    case 8 : b+=k[1]; a+=k[0]; break;
-    case 7 : b+=k[1]<<8; a+=k[0]; break;
-    case 6 : b+=k[1]<<16; a+=k[0]; break;
-    case 5 : b+=k[1]<<24; a+=k[0]; break;
-    case 4 : a+=k[0]; break;
-    case 3 : a+=k[0]<<8; break;
-    case 2 : a+=k[0]<<16; break;
-    case 1 : a+=k[0]<<24; break;
-    case 0 : return c;              /* zero length strings require no mixing */
-    }
-
-  } else {                        /* need to read the key one byte at a time */
-    register uint8_t *k = key;
-
-    /*--------------- all but the last block: affect some 32 bits of (a,b,c) */
-    while (length > 12)
-    {
-      a += ((uint32_t)k[0])<<24;
-      a += ((uint32_t)k[1])<<16;
-      a += ((uint32_t)k[2])<<8;
-      a += ((uint32_t)k[3]);
-      b += ((uint32_t)k[4])<<24;
-      b += ((uint32_t)k[5])<<16;
-      b += ((uint32_t)k[6])<<8;
-      b += ((uint32_t)k[7]);
-      c += ((uint32_t)k[8])<<24;
-      c += ((uint32_t)k[9])<<16;
-      c += ((uint32_t)k[10])<<8;
-      c += ((uint32_t)k[11]);
-      mix(a,b,c);
-      length -= 12;
-      k += 12;
-    }
-
-    /*-------------------------------- last block: affect all 32 bits of (c) */
-    switch(length)                   /* all the case statements fall through */
-    {
-    case 12: c+=((uint32_t)k[11])<<24;
-    case 11: c+=((uint32_t)k[10])<<16;
-    case 10: c+=((uint32_t)k[9])<<8;
-    case 9 : c+=k[8];
-    case 8 : b+=((uint32_t)k[7])<<24;
-    case 7 : b+=((uint32_t)k[6])<<16;
-    case 6 : b+=((uint32_t)k[5])<<8;
-    case 5 : b+=k[4];
-    case 4 : a+=((uint32_t)k[3])<<24;
-    case 3 : a+=((uint32_t)k[2])<<16;
-    case 2 : a+=((uint32_t)k[1])<<8;
-    case 1 : a+=k[0];
-             break;
-    case 0 : return c;
-    }
-  }
-
-  final(a,b,c);
-  return c;
-}
-
-Obj FuncJenkinsHashInOrb(Obj self, Obj x, Obj offset, Obj bytelen, Obj hashlen)
-{
-   void *input;
-   uint32_t len;
-   uint32_t key;
-   uint32_t init = 0;
-   uint32_t mod;
-   
-   input = (void *)((UChar *) ADDR_OBJ(x) + INT_INTOBJ(offset));
-   len = (uint32_t)INT_INTOBJ(bytelen);
-   mod = INT_INTOBJ(hashlen);
-	
-// Take advantage of endianness if possible
-#ifdef WORDS_BIGENDIAN
-   key = hashbig(input, len, init);
-#else
-   key = hashlittle(input, len, init);
-#endif
-   
-   return INTOBJ_INT((Int)(key % mod + 1));
-}
-
 Obj FuncPermList(Obj self, Obj list);
 
 Obj FuncMappingPermSetSet(Obj self, Obj src, Obj dst)
@@ -1930,96 +1398,19 @@ Obj FuncMappingPermSetSet(Obj self, Obj src, Obj dst)
     }
     return FuncPermList(self,out);
 } 
- 
-#define DEGREELIMITONSTACK 512
 
-Obj FuncMappingPermListList(Obj self, Obj src, Obj dst)
-{
-    Int l;
-    Int i;
-    Int d;
-    Int next;
-    Obj out;
-    Obj tabdst, tabsrc;
-    Int x;
-    Int mytabs[DEGREELIMITONSTACK+1];
-    Int mytabd[DEGREELIMITONSTACK+1];
+Obj HASH_FUNC_FOR_BLIST (Obj self, Obj blist, Obj data_gap) {
 
-    l = LEN_LIST(src);
-    if (l != LEN_LIST(dst)) {
-        ErrorReturnVoid( "both arguments must be lists of equal length", 
-                     0L, 0L, "type 'return;' or 'quit;' to exit break loop" );
-        return 0L;
-    }
-    d = 0;
-    for (i = 1;i <= l;i++) {
-        x = INT_INTOBJ(ELM_LIST(src,i));
-        if (x > d) d = x;
-    }
-    for (i = 1;i <= l;i++) {
-        x = INT_INTOBJ(ELM_LIST(dst,i));
-        if (x > d) d = x;
-    }
-    if (d <= DEGREELIMITONSTACK) {
-        /* Small case where we work on the stack: */
-        memset(&mytabs,0,sizeof(mytabs));
-        memset(&mytabd,0,sizeof(mytabd));
-        for (i = 1;i <= l;i++) {
-            mytabs[INT_INTOBJ(ELM_LIST(src,i))] = i;
-        }
-        for (i = 1;i <= l;i++) {
-            mytabd[INT_INTOBJ(ELM_LIST(dst,i))] = i;
-        }
-        out = NEW_PLIST(T_PLIST_CYC,d);
-        SET_LEN_PLIST(out,d);
-        /* No garbage collection from here ... */
-        next = 1;
-        for (i = 1;i <= d;i++) {
-            if (mytabs[i]) {   /* if i is in src */
-                SET_ELM_PLIST(out,i, ELM_LIST(dst,mytabs[i]));
-            } else {
-                /* Skip things in dst: */
-                while (mytabd[next]) next++;
-                SET_ELM_PLIST(out,i,INTOBJ_INT(next));
-                next++;
-            }
-        }
-        /* ... to here! No CHANGED_BAG needed since this is a new object! */
-    } else {
-        /* Version with intermediate objects: */
+  size_t res  = 0;
+  UInt   nr  = NUMBER_BLOCKS_BLIST(blist);
+  UInt*  ptr  = BLOCKS_BLIST(blist);
+  UInt   data = INT_INTOBJ(data_gap);
 
-        tabsrc = NEW_PLIST(T_PLIST,d);
-        SET_LEN_PLIST(tabsrc,0);
-        /* No garbage collection from here ... */
-        for (i = 1;i <= l;i++) {
-            SET_ELM_PLIST(tabsrc,INT_INTOBJ(ELM_LIST(src,i)),INTOBJ_INT(i));
-        }
-        /* ... to here! No CHANGED_BAG needed since this is a new object! */
-        tabdst = NEW_PLIST(T_PLIST,d);
-        SET_LEN_PLIST(tabdst,0);
-        /* No garbage collection from here ... */
-        for (i = 1;i <= l;i++) {
-            SET_ELM_PLIST(tabdst,INT_INTOBJ(ELM_LIST(dst,i)),INTOBJ_INT(i));
-        }
-        /* ... to here! No CHANGED_BAG needed since this is a new object! */
-        out = NEW_PLIST(T_PLIST_CYC,d);
-        SET_LEN_PLIST(out,d);
-        /* No garbage collection from here ... */
-        next = 1;
-        for (i = 1;i <= d;i++) {
-            if (ELM_PLIST(tabsrc,i)) {   /* if i is in src */
-                SET_ELM_PLIST(out,i,
-                    ELM_LIST(dst,INT_INTOBJ(ELM_PLIST(tabsrc,i))));
-            } else {
-                /* Skip things in dst: */
-                while (ELM_PLIST(tabdst,next)) next++;
-                SET_ELM_PLIST(out,i,INTOBJ_INT(next));
-                next++;
-            }
-        }
-        /* ... to here! No CHANGED_BAG needed since this is a new object! */
-    }
-    return FuncPermList(self,out);
+  while (nr > 0) {
+    res = (res * 23) + *(ptr++);
+    nr--;
+  }
+  return INTOBJ_INT((res % data) + 1);
 }
 
 /*F * * * * * * * * * * * * * initialize package * * * * * * * * * * * * * * */
@@ -2101,21 +1492,13 @@ static StructGVarFunc GVarFuncs [] = {
     HTUpdate_TreeHash_C,
     "orb.c:HTUpdate_TreeHash_C" },
 
-  { "GenericHashFunc_C", 2, "x, data",
-    GenericHashFunc_C,
-    "orb.c:GenericHashFunc_C" }, 
-
-  { "JENKINS_HASH_IN_ORB", 4, "x, offset, bytelen, hashlen",
-    FuncJenkinsHashInOrb, 
-    "pkg/orb/src/orb.c:JENKINS_HASH_IN_ORB" },
-
   { "MappingPermSetSet_C", 2, "src, dst",
     FuncMappingPermSetSet,
     "pkg/orb/src/orb.c:FuncMappingPermSetSet_C" },
 
-  { "MappingPermListList_C", 2, "src, dst",
-    FuncMappingPermListList,
-    "pkg/orb/src/orb.c:FuncMappingPermListList" },
+  { "HASH_FUNC_FOR_BLIST", 2, "blist, data",
+    HASH_FUNC_FOR_BLIST,
+    "pkg/orb/src/orb.c:HASH_FUNC_FOR_BLIST" },
 
   { 0 }
 
@@ -2194,9 +1577,10 @@ StructInitInfo * Init__orb ( void )
 
 /*
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; version 2 of the License.
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -2204,8 +1588,7 @@ StructInitInfo * Init__orb ( void )
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
